@@ -24,6 +24,33 @@ from cbmc_viewer import util
 
 JSON_TAG = 'viewer-coverage'
 
+################################################################
+# Incomplete coverage reporting
+#
+# CBMC coverage output may included malformed data that can't be
+# parsed and must be skipped.  Skipping coverage data may make the
+# coverage report incorrect.  For example, skipping coverage data
+# saying that some lines have been missed may cause the coverage
+# reported to be too high.
+#
+# This use of global state is not good python.
+
+INCOMPLETE_COVERAGE = False
+
+def set_incomplete_coverage():
+    global INCOMPLETE_COVERAGE
+    INCOMPLETE_COVERAGE = True
+
+def clear_incomplete_coverage():
+    global INCOMPLETE_COVERAGE
+    INCOMPLETE_COVERAGE = False
+
+def warn_of_incomplete_coverage(path):
+    global INCOMPLETE_COVERAGE
+    if INCOMPLETE_COVERAGE:
+        logging.warning("Skipping malformed coverage data in %s.", path)
+        logging.warning("Use the --verbose to see what coverage data was skipped.")
+
 ################################################################
 # Line coverage status
 
@@ -363,6 +390,7 @@ def load_cbmc_json(json_file, root):
         raise UserWarning(
             f"Failed to locate coverage goals in json coverage data: {json_file}") from error
 
+    clear_incomplete_coverage()
     coverage = {}
     for goal in goals:
         lines = (parse_basicBlockLines(goal.get("basicBlockLines")) or
@@ -370,6 +398,7 @@ def load_cbmc_json(json_file, root):
         status = goal["status"]
         wkdir = srcloct.json_srcloc_wkdir(goal["sourceLocation"])
         coverage = add_coverage_data(coverage, lines, status, wkdir, root)
+    warn_of_incomplete_coverage(json_file)
 
     try:
         RAW_COVERAGE_DATA(coverage)
@@ -397,6 +426,7 @@ def load_cbmc_xml(xml_file, root):
     if not xml or xml is None:   # Why is 'xml is None' required?
         raise UserWarning(f"Failed to load xml coverage data: {xml_file}")
 
+    clear_incomplete_coverage()
     coverage = {}
     for goal in xml.iter("goal"):
         lines = (parse_basic_block_lines(goal.find("basic_block_lines")) or
@@ -404,6 +434,7 @@ def load_cbmc_xml(xml_file, root):
         status = goal.get("status")
         wkdir = srcloct.xml_srcloc_wkdir(goal.find("location"))
         coverage = add_coverage_data(coverage, lines, status, wkdir, root)
+    warn_of_incomplete_coverage(xml_file)
 
     try:
         RAW_COVERAGE_DATA(coverage)
@@ -481,93 +512,113 @@ def update_coverage(coverage, path, func, line, status):
 # subelements of the goal in CBMC's xml or json coverage output.
 
 def parse_lines(string):
-    """Extract line numbers from the string encoding of line numbers used in coverage output"""
+    """The line numbers denoted by a line number encoding found in coverage data"""
 
     # string is an encoding of a set of line numbers like
     #   "1,3,6" -> {1, 3, 6}
     #   "1-3,6,20-32" -> {1, 2, 3, 6, 20, 21, 22}
 
-    lines = set()
-    for group in string.split(','):
-        bounds = group.split('-')
-        if len(bounds) == 1:
-            lines.add(int(bounds[0]))
-        elif len(bounds) == 2:
-            lines.update(range(int(bounds[0]), int(bounds[1])+1))
-        else:
-            raise UserWarning(f"Unexpected encoding of line numbers: {string}")
-    return sorted(lines)
+    try:
+        lines = set()
+        for group in string.split(','):
+            bounds = group.split('-')
+            if len(bounds) == 1:
+                lines.add(int(bounds[0]))
+            elif len(bounds) == 2:
+                lines.update(range(int(bounds[0]), int(bounds[1])+1))
+            else:
+                raise ValueError
+        return sorted(lines)
+    except ValueError:
+        # ValueError: invalid literal for int()
+        logging.info("Skipping malformed line number encoding: %s", string)
+        set_incomplete_coverage()
+        return []
 
 def parse_description(description):
-    """Extract basic block source lines from a coverage goal's textual description"""
+    """The source locations in the basic block encoded by a coverage goal description"""
 
     try:
         # description is "block N (lines BASIC_BLOCK)"
         basic_block = re.match(r'block [0-9]+ \(lines (.*)\)', description).group(1)
 
+        if basic_block is None:
+            raise ValueError
+
         # basic_block is
         #   chunk1;chunk2;chunk3
         # each chunk is
         #   test.c:foo:3,6-10
-        # each function name foo from rust may include embedded semicolons like
-        #   main::foo
-        lines = [(fyle, func, line)
-                 for chunk in basic_block.split(';')
-                 for fyle, func_lines in [chunk.split(':', 1)]  # fyle:func:lines -> fyle,func:lines
-                 for func, lines in [func_lines.rsplit(':', 1)] # func:lines -> func,lines
-                 for line in parse_lines(lines)]
-        assert all(fyle and func and line for fyle, func, line in lines)
-        return lines
-    except (AttributeError, ValueError, AssertionError) as error:
-        # AttributeError after match(): 'NoneType' object has no attribute 'group'
+        # each function name foo from rust may include embedded semicolons like main::foo
+
+        srclocs = []
+        for chunk in basic_block.split(';'):
+            fyle, func_lines = chunk.split(':', 1)  # fyle:func:lines -> fyle,func:lines
+            func, lines = func_lines.rsplit(':', 1) # func:lines -> func,lines
+            for line in parse_lines(lines):
+                if fyle and func and line:
+                    lines.append((fyle, func, line))
+                else:
+                    logging.info(
+                        'Skipping malformed source location in coverage goal description: %s: '
+                        'Found file:%s function:%s line:%s', description, fyle, func, line
+                    )
+                    set_incomplete_coverage()
+        return srclocs
+    except ValueError:
         # ValueError after after split()/rsplit(): not enough values to unpack
-        # ValueError in parse_lines(): invalid literal for int()
-        raise UserWarning(f'Unexpected coverage goal description: "{description}"') from error
+        logging.info('Skipping malformed coverage goal description: %s', description)
+        set_incomplete_coverage()
+        return []
 
 def parse_basic_block_lines(basic_block_lines):
-    """Extract basic block source lines from a coverage goal's xml subelement"""
+    """The source locations in a basic block of an xml coverage goal"""
+
+    # basic_block_lines is xml
+    #   <basic_block_lines>
+    #     <line file="test.c" function="foo">3,6-10</line>
+    #     ...
+    #   </basic_block_lines>
 
     if basic_block_lines is None:
         return []
 
-    try:
-        # basic_block_lines is xml
-        #   <basic_block_lines>
-        #     <line file="test.c" function="foo">3,6-10</line>
-        #     ...
-        #   </basic_block_lines>
-        lines = [(fyle, func, line)
-                 for bbl in basic_block_lines.iter("line")
-                 for fyle, func, lines in [(bbl.get("file"), bbl.get("function"), bbl.text)]
-                 for line in parse_lines(lines)]
-        assert all(fyle and func and line for fyle, func, line in lines)
-        return lines
-    except (ValueError, AssertionError) as error:
-        # ValueError in parse_lines(): invalid literal for int()
-        raise UserWarning(f'Unexpected coverage goal xml data: "{basic_block_lines}"') from error
+    srclocs = []
+    for bbl in basic_block_lines.iter("line"):
+        fyle, func, lines = bbl.get("file"), bbl.get("function"), bbl.text
+        for line in parse_lines(lines):
+            if fyle and func and line:
+                srclocs.append((fyle, func, line))
+            else:
+                logging.info('Skipping malformed source location in coverage goal: '
+                             'Found file:%s function:%s line:%s', fyle, func, line)
+                set_incomplete_coverage()
+    return srclocs
 
 def parse_basicBlockLines(basicBlockLines): # pylint: disable=invalid-name
-    """Extract basic block source lines from a coverage goal's json data"""
+    """The source locations in a basic block of a json coverage goal"""
+
+    # basicBlockLines is json
+    #   "basicBlockLines": {
+    #     "test.c": {
+    #       "main": "16,17"
+    #     }
+    #   }
 
     if basicBlockLines is None:
         return []
 
-    try:
-        # basicBlockLines is json
-        #   "basicBlockLines": {
-        #     "test.c": {
-        #       "main": "16,17"
-        #     }
-        #   }
-        lines = [(fyle, func, line)
-                 for fyle, fyle_data in basicBlockLines.items()
-                 for func, lines in fyle_data.items()
-                 for line in parse_lines(lines)]
-        assert all(fyle and func and line for fyle, func, line in lines)
-        return lines
-    except (ValueError, AssertionError) as error:
-        # ValueError in parse_lines(): invalid literal for int()
-        raise UserWarning(f'Unexpected coverage goal json data: "{basicBlockLines}"') from error
+    srclocs = []
+    for fyle, fyle_data in basicBlockLines.items():
+        for func, lines in fyle_data.items():
+            for line in parse_lines(lines):
+                if fyle and func and line:
+                    srclocs.append((fyle, func, line))
+                else:
+                    logging.info('Skipping malformed source location in coverage goal: '
+                                 'Found file:%s function:%s line:%s', fyle, func, line)
+                    set_incomplete_coverage()
+    return srclocs
 
 ################################################################
 # Parse the hit/miss status in a coverage goal
